# -*- coding: utf-8 -*-

import fnmatch
import os  # noqa E402

from setuptools import Extension, find_packages, setup
from setuptools.command.build_py import build_py as build_py_orig

from lambert_lm.embedding.layout.numba_utils import cc
from setup import (NAME, DESCRIPTION, URL, REQUIRES_PYTHON, VERSION,
                   REQUIRES, EXTRAS)


# Exclude files that actually should be human (or python) readable
cython_excludes = [
    "*/__version__.py",
    "**/__main__.py",
    "**/__init__.py",
    "lambert_lm/embedding/layout/numba_utils.py",
]

extensions = [Extension("lambert_lm.*", ["lambert_lm/**/*.py"], extra_compile_args=['-g0'])]

ext_modules = [cc.distutils_extension()]

# Due to interactions between Cython and distutils, cythonize import must be after cc.distutils()
from Cython.Build import cythonize

ext_modules += cythonize(
    extensions,
    build_dir='build',
    compiler_directives=dict(
        always_allow_keywords=True,
        language_level='3'
    ),
    exclude=cython_excludes,
)



# This method is taken from here
# https://stackoverflow.com/a/56043918
def not_cythonized(data):
    (package, module, filepath) = data
    return any(
        fnmatch.fnmatchcase(filepath, pat=pattern)
        for pattern in cython_excludes
    ) or not any(
        fnmatch.fnmatchcase(filepath, pat=pattern)
        for ext in extensions
        for pattern in ext.sources
    )


class build_py(build_py_orig):
    """This overwrites a default build_py method.

    This only finds non-cythonized .py files and MANIFEST files.

    """

    def find_modules(self):
        modules = super().find_modules()
        return list(filter(not_cythonized, modules))

    def find_package_modules(self, package, package_dir):
        modules = super().find_package_modules(package, package_dir)
        return list(filter(not_cythonized, modules))


setup(
    name=NAME,
    ext_modules=ext_modules,
    version=VERSION,
    description=DESCRIPTION,
    packages=['lambert_lm'] + ['lambert_lm.' + pkg for pkg in find_packages('lambert_lm')],
    url=URL,
    license='PROPRIETARY Applica',
    install_requires=REQUIRES,
    extras_require=EXTRAS,
    python_requires=REQUIRES_PYTHON,
    cmdclass=dict(
        build_py=build_py,
    ),
)
