# -*- coding: utf-8 -*-

import unittest

from due_evaluator.scorers.fscorer import FScorer


class TestFscorer(unittest.TestCase):
    """Tests for FScorer class."""

    def from_items(self, vals):
        anns = []
        for key, val in vals:
            anns.append({
                'key': key,
                'values': [
                    {'value': val, 'value_variants': val}
                ]
            })
        return {
            'name': 'test',
            'annotations': anns
        }

    def test_all(self):
        """Test all methods together."""
        fscorer = FScorer()
        out_items = self.from_items([
            ('are_counterparts', 'yes'),
            ('effective_date', '2011-11-08'),
            ('jurisdiction', 'New York'),
            ('first_party', 'JDA Software Group, Inc.'),
            ('second_party', 'Red Prairie Holding, Inc.'),
        ])
        ref_items = self.from_items([
            ('are_counterparts', 'yes'),
            ('effective_date', '2012-09-04'),
            ('first_party', 'JDA Software Group, Inc.'),
            ('jurisdiction', 'Delaware'),
            ('second_party', 'Red Prairie Holding, Inc.'),
            ('term' , '2 years'),
        ])
        fscorer.add(out_items, ref_items)
        self.assertAlmostEqual(fscorer.precision(), 3 / 5)
        self.assertAlmostEqual(fscorer.recall(), 3 / 6)
        self.assertAlmostEqual(fscorer.f_score(), 6 / 11)

    def test_empty_fscorer(self):
        """Test empty scorer."""
        fscorer = FScorer()
        self.assertAlmostEqual(fscorer.precision(), 0)
        self.assertAlmostEqual(fscorer.recall(), 0)
        self.assertAlmostEqual(fscorer.f_score(), 0)

